#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
"""
Tests for casa_envoy.palaver
"""
import argparse
from unittest.mock import MagicMock, patch

import casa_envoy.palaver as palaver
from casa_envoy.launchers import CalibrationLauncher, ImagingLauncher
from pycapo import CapoConfig

expected_settings = {
    "useCasa": False,
    "homeForReprocessing": "/home/casa/packages/pipeline/current",
    "rootDirectory": "/lustre/aoc/cluster/pipeline/docker/workspaces/spool",
    "processingDirectory": "tmpo1ca1pp_",
    "parent_path": "/lustre/aoc/cluster/pipeline/docker/workspaces/spool/tmpo1ca1pp_",
    "metadata": "test/input_files/test.json",
    "ppr": "test/input_files/PPR.xml",
    "product_type": "standard-cal",
}
args = argparse.Namespace()


class TestPalaver:
    def test_get_settings(self):
        """
        verifies correct determination of needed settings

        :return:
        """
        args.standard_cal = ["test/input_files/test.json", "test/input_files/PPR.xml"]
        args.restore = False

        with patch(
            "casa_envoy.palaver.CapoConfig",
            MagicMock(return_value=CapoConfig(profile="test-capo", path="test/input_files/")),
        ):
            with patch(
                "pathlib.Path.cwd",
                MagicMock(return_value="/lustre/aoc/cluster/pipeline/docker/workspaces/spool/tmpo1ca1pp_"),
            ) as cwd:
                settings = palaver._get_settings(cwd, args.standard_cal)
                assert settings["useCasa"] == expected_settings["useCasa"]
                assert settings["homeForReprocessing"] == expected_settings["homeForReprocessing"]
                assert settings["metadata"] == expected_settings["metadata"]
                assert settings["ppr"] == expected_settings["ppr"]

        args.standard_cal = None

    @patch("json.loads")
    def test_setup_launcher(self, mock_json):
        """
        Verifies that the correct type of launcher is created

        :return:
        """
        with patch(
            "pathlib.Path.cwd",
            MagicMock(return_value="/lustre/aoc/cluster/pipeline/docker/workspaces/spool/tmpo1ca1pp_"),
        ) as cwd:
            args.parallel = None

            args.standard_cal = ["test/input_files/test.json", "test/input_files/PPR.xml"]
            args.vlass_cal = None
            args.restore = False
            launcher = palaver._setup_launcher(cwd, args)
            assert isinstance(launcher, CalibrationLauncher)
            args.standard_cal = None

            args.standard_img = [
                "test/input_files/image-metadata.json",
                "test/input_files/cmsimage-PPR.xml",
            ]
            args.vlass_img = None
            args.integrated = False
            launcher = palaver._setup_launcher(cwd, args)
            assert isinstance(launcher, ImagingLauncher)
            args.standard_img = None

    def test_parser(self):
        """
        Verifies parser setup

        :return:
        """
        parser = palaver.arg_parser()
        assert isinstance(parser, argparse.ArgumentParser)

    @patch("json.loads")
    @patch("os.chdir")
    @patch("os.getcwd")
    def test_main_cal(self, mock_cwd, mock_chdir, mock_json):
        """
        verifies use of correct launcher for calibrations

        :param mock_cwd:
        :param mock_chdir:
        :param mock_json:
        :return:
        """
        args.standard_cal = ["test/input_files/test.json", "test/input_files/PPR.xml"]
        args.integrated = False
        args.vlass_cal = None
        args.parallel = None

        with patch("argparse.ArgumentParser.parse_args", MagicMock(return_value=args)) as mock_args:
            with patch("casa_envoy.launchers.CalibrationLauncher.launch_casa") as cal_launcher:
                palaver.main()
                assert cal_launcher.call_count == 1

        args.standard_cal = None

    @patch("json.loads")
    @patch("os.chdir")
    @patch("os.getcwd")
    def test_main_img(self, mock_cwd, mock_chdir, mock_json):
        """
        verifies use of correct launcher for images

        :param mock_cwd:
        :param mock_chdir:
        :param mock_json:
        :return:
        """
        args.standard_img = [
            "test/input_files/image-metadata.json",
            "test/input_files/cmsimage-PPR.xml",
        ]
        args.vlass_img = None
        args.parallel = None

        with patch("argparse.ArgumentParser.parse_args", MagicMock(return_value=args)) as mock_args:
            with patch("casa_envoy.launchers.ImagingLauncher.launch_casa") as img_launcher:
                palaver.main()
                assert img_launcher.call_count == 1

        args.standard_img = None
